@@ -3,7 +3,7 @@ use warnings;
 package GeneratePackage;
 # vim:ts=8:sw=2:et:sta:sts=2
 
-use base 'Exporter';
+our @ISA = ('Exporter');
 our @EXPORT = qw(tmpdir generate_file);
 
 use Cwd;
@@ -12,14 +12,24 @@ use File::Path;
 use File::Temp;
 use IO::File;
 
+BEGIN {
+  my $cwd = File::Spec->rel2abs(Cwd::cwd);
+  sub _original_cwd { return $cwd }
+}
+
 sub tmpdir {
+  my (@args) = @_;
   File::Temp::tempdir(
     'MMD-XXXXXXXX',
     CLEANUP => 0,
-    DIR => ($ENV{PERL_CORE} ? File::Spec->rel2abs(Cwd::cwd) : File::Spec->tmpdir),
+    DIR => ($ENV{PERL_CORE} ? _original_cwd : File::Spec->tmpdir),
+    @args,
   );
 }
 
+my $tmp;
+BEGIN { $tmp = tmpdir; Test::More::note "using temp dir $tmp"; }
+
 sub generate_file {
   my ($dir, $rel_filename, $content) = @_;
 
@@ -35,4 +45,12 @@ sub generate_file {
   return $abs_filename;
 }
 
+END {
+  die "tests failed; leaving temp dir $tmp behind"
+    if $ENV{AUTHOR_TESTING} and not Test::Builder->new->is_passing;
+  Test::More::note "removing temp dir $tmp";
+  chdir _original_cwd;
+  File::Path::rmtree($tmp);
+}
+
 1;
